/*
 *      gpg-crypter.c
 *
 *      Copyright 2006 Florian Sievers <florian@dopehouse.dyndns.org>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; version 2 of the License.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif
#ifdef HAVE_LOCALE_H
	#include <locale.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gpgme.h>
#include <gtk/gtk.h>

#include "gpg-crypter.h"
#include "error-dialogs.h"
#include "support.h"

static void ptr_array_cleanup(gpointer data, gpointer user_data);
void passphrase_entry_hit_enter(GtkWidget *widget, gpointer data);


/* get all gpg-keys and insert them into app_data->key_array[] */
int gpg_get_keys(application_data *app_data)
{
	gpgme_error_t err;
	gpgme_key_t tmp_key[2];
	int get_keys_running = 1;
	gchar *tmp_key_id_string;
	gchar *tmp_key_string;
	int IDX;
	int tmp_key_string_length;

	gchar *new_key;

	tmp_key[1] = 0;

	err = gpgme_op_keylist_start (app_data->CTX, "", 0);
	if(err)
	{
		printf("2: %s\n", gpgme_strerror(err));
	}

	while(get_keys_running != 0)
	{
		err = gpgme_op_keylist_next(app_data->CTX, tmp_key);
		switch(gpgme_err_code(err))
		{
			case GPG_ERR_EOF: // all keys collected
				get_keys_running = 0;
				continue;
				break;

			case GPG_ERR_NO_ERROR:
				app_data->num_keys += 1;
				tmp_key_string = g_strconcat((tmp_key[0]->uids->name), " (", (tmp_key[0]->uids->comment), ")\n    ", (tmp_key[0]->uids->email), NULL);
				g_ptr_array_add(app_data->key_string_array, tmp_key_string);
				tmp_key_id_string = g_strconcat((tmp_key[0]->uids->uid), NULL);
				g_ptr_array_add(app_data->key_array, tmp_key_id_string);
				break;

			default:
				get_keys_running = 0;
				continue;
				break;
		}
	}

	return 0;
}

/* initialize gpgme */
int gpg_init(application_data *app_data, gpgme_protocol_t proto)
{
	gpgme_error_t err;

	app_data->key[1]=0;
	app_data->key_array = g_ptr_array_new();
	app_data->key_string_array = g_ptr_array_new();

	app_data->ASCIImode = 0;

	// Allocate GStrings
	app_data->plaintext_file = g_string_new('\0');
	app_data->cipher_file = g_string_new('\0');

	/* Initialize the locale environment.  */
#ifdef HAVE_LC_MESSAGES
	setlocale (LC_ALL, "");
	gpgme_check_version (NULL);
	gpgme_set_locale (NULL, LC_CTYPE, setlocale (LC_CTYPE, NULL));
	gpgme_set_locale (NULL, LC_MESSAGES, setlocale (LC_MESSAGES, NULL));
#endif

	err = gpgme_engine_check_version (proto);
	if(err)
	{
		error_dialog_with_string(_("GnuPG (OpenPGP) not found!\n"));
		return 1;
	}

	// Create context
	err = gpgme_new(&(app_data)->CTX);
	if(err)
	{
		printf("1: %s\n", gpgme_strerror(err));
	}

	// Set ASCII armor to yes
	gpgme_set_armor(app_data->CTX, 1);

	/* register passphrase callback */
	gpgme_set_passphrase_cb(app_data->CTX, gpg_passphrase_callback, app_data);
	gpg_get_keys(app_data);

	/* register progress callback */
	gpgme_set_progress_cb(app_data->CTX, gpg_progress_callback, NULL);

	return 0;
}

/* release all resources that were allocated for gpgme and app_data */
int gpg_release(application_data *app_data)
{
	/* disable callbacks */
	gpgme_set_passphrase_cb(app_data->CTX, NULL, NULL);
	gpgme_set_progress_cb (app_data->CTX, NULL, NULL);

	// Release context
	gpgme_release(app_data->CTX);

	/* cleanup key_array and key_string_array */
	g_ptr_array_foreach(app_data->key_array, ptr_array_cleanup, NULL);
	g_ptr_array_foreach(app_data->key_string_array, ptr_array_cleanup, NULL);
	g_ptr_array_free(app_data->key_array, FALSE);
	g_ptr_array_free(app_data->key_string_array, FALSE);

	g_string_free(app_data->plaintext_file, TRUE);
	g_string_free(app_data->cipher_file, TRUE);

	return 0;
}

/* free allocated memory for g_prt_arrays */
static void ptr_array_cleanup(gpointer data, gpointer user_data)
{
	const gchar *data_string;

	data_string = (gchar *) data;

	free(data);

	return;
}

/* gpgme passphrase callback dialog */
gpgme_error_t gpg_passphrase_callback(void *HOOK, const char *UID_HINT, const char *PASSPHRASE_INFO, int PREV_WAS_BAD, int fd)
{
	GtkWidget *passphrase_dialog;
	GtkWidget *entry_passphrase;
	gint result;
	//const gchar *passphrase = 0;
	const char nl = '\n';

	application_data *app_data;

	app_data = (application_data *) HOOK;

	/*if(PREV_WAS_BAD == 0)
	{
		write(fd, app_data->passphrase, strlen(app_data->passphrase));
		write(fd, &nl, 1);
		return GPG_ERR_NO_ERROR;
	}*/

	passphrase_dialog = create_passphrase_callback_dialog();

	result = gtk_dialog_run(GTK_DIALOG(passphrase_dialog));

	entry_passphrase = (GtkWidget *) lookup_widget(passphrase_dialog, "passphrase_callback_entry");

	switch (result)
	{
		case GTK_RESPONSE_OK:
			app_data->passphrase = gtk_entry_get_text(GTK_ENTRY(entry_passphrase));

			write(fd, app_data->passphrase, strlen(app_data->passphrase));
			write(fd, &nl, 1);

			gtk_widget_destroy(GTK_WIDGET(passphrase_dialog));
			return GPG_ERR_NO_ERROR;
			break;

		case GTK_RESPONSE_CANCEL:
			write(fd, &nl, 1);
			gtk_widget_destroy(GTK_WIDGET(passphrase_dialog));
			return GPG_ERR_CANCELED;
			break;

		default:
			//do_nothing_since_dialog_was_cancelled ();
			write(fd, &nl, 1);
			gtk_widget_destroy(GTK_WIDGET(passphrase_dialog));
			return GPG_ERR_CANCELED;
			break;
	}

	return GPG_ERR_CANCELED;
}

/* insert a new entry to the combobox */
void insert_combobox_row(gpointer data, gpointer user_data)
{
	const gchar *key_string;
	GtkWidget *widget;
	key_string = (gchar *) data;
	widget = (GtkWidget *) user_data;

	/* append data into combobox */
	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), g_locale_to_utf8(key_string, -1, NULL, NULL, NULL));

	return;
}

/* select app_data->key_array[] where the index is the selected combobox entry */
void combo_select_key(GtkWidget *widget, gpointer data)
{
	gpgme_error_t err;
	gint selected_key;

	/* get the number of the selected entry in the combobox */
	selected_key = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

	err = gpgme_op_keylist_start (app_data->CTX, g_ptr_array_index(app_data->key_array, selected_key), 0);
	if(err)
	{
		printf("2: %s\n", gpgme_strerror(err));
	}

	err = gpgme_op_keylist_next(app_data->CTX, app_data->key);
	if(err)
	{
		printf("2: %s\n", gpgme_strerror(err));
	}

	return;
}

/* clear th plaintext buffer */
void clear_plaintext_text_buffer(GtkWidget *widget, gpointer data)
{
	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(app_data->buffer_plaintext), "", -1);

	return;
}

/* clear th cipher buffer */
void clear_cipher_text_buffer(GtkWidget *widget, gpointer data)
{
	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(app_data->buffer_cipher), "", -1);

	return;
}

/* change ascii-armor for files */
void change_chipher_file_asciimode(GtkWidget *widget, gpointer data)
{
	application_data *app_data;

	app_data = (application_data *) data;

	switch(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
	{
		case TRUE:
			app_data->ASCIImode = 1; // ASCII-Armor on
			break;
		case FALSE:
			app_data->ASCIImode = 0; // ASCII-Armor off
			break;
	}

	return;
}

void plaintext_file_changed(GtkWidget *widget, gpointer data)
{
	application_data *app_data;

	app_data = (application_data *) data;


	g_string_printf(app_data->plaintext_file, "%s", gtk_entry_get_text(GTK_ENTRY(widget)));

	return;
}

void cipher_file_changed(GtkWidget *widget, gpointer data)
{
	application_data *app_data;

	app_data = (application_data *) data;

	g_string_printf(app_data->cipher_file, "%s", gtk_entry_get_text(GTK_ENTRY(widget)));

	return;
}

void plaintext_file_selector(GtkWidget *widget, gpointer data)
{
	GtkWidget *file_chooser;
	GtkWidget *entry;
	gchar *filename;

	entry = lookup_widget(widget, "plaintext_file_entry");

	file_chooser = gtk_file_chooser_dialog_new(_("Choose plaintext file"), NULL, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(file_chooser), TRUE);
	gtk_dialog_run(GTK_DIALOG(file_chooser));

	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
	gtk_entry_set_text(GTK_ENTRY(entry), filename);

	gtk_widget_destroy(file_chooser);

	return;
}

void cipher_file_selector(GtkWidget *widget, gpointer data)
{
	GtkWidget *file_chooser;
	GtkWidget *entry;
	gchar *filename;

	entry = lookup_widget(widget, "cipher_file_entry");

	file_chooser = gtk_file_chooser_dialog_new(_("Choose cipher file"), NULL, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(file_chooser), TRUE);
	gtk_dialog_run(GTK_DIALOG(file_chooser));

	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
	gtk_entry_set_text(GTK_ENTRY(entry), filename);

	gtk_widget_destroy(file_chooser);

	return;
}

void gpg_progress_callback(void *status, const char *what, int type, int current, int total)
{
	/* Waiting for io_callback implemtation */
}

